/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.service.impl;

import java.io.File;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.BodyPart;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.context.ApplicationContextFactory;
import org.jdesigner.platform.jndi.ContextFactory;
import org.jdesigner.platform.util.StringUtils;

import org.jdesigner.application.template.service.CommonServiceException;
import org.jdesigner.application.template.service.CommonServiceProperties;
import org.jdesigner.application.template.service.MailService;

import freemarker.template.Configuration;
import freemarker.template.ObjectWrapper;
import freemarker.template.Template;
import freemarker.template.TemplateExceptionHandler;

/**
 * EmailServiceImpl
 * 
 * @author SangBoo Lee
 * @date 2009. 11. 16.
 * @since 1.0
 * 
 */
public class MailServiceImpl implements MailService {

	private static final Log log = LogFactory.getLog(MailServiceImpl.class);

	private static final String MAIL_ENCODING = "UTF-8";
	private static final String SEND_SESSION_JNDI = "service.mail.session.jndi";
	private static final String MAIL_TEMLATE_DIR = "service.mail.template.dir";
	private static final String MAIL_STORE_NAME = "service.mail.store";
	private static final String MAIL_DOMAIN = "service.mail.domain";

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.MailService#sendTemplateMail (java.lang.String[], java.lang.String[], java.lang.String,
	 * java.lang.String, java.lang.String, java.lang.String, java.util.Map, boolean, java.util.List, int, boolean)
	 */
	public void sendTemplateMail(String[] to, String[] cc, String from, String subject, String templateFileName, Map map, boolean isHTML, List attachments, int priority, boolean saveStore)
			throws CommonServiceException {

		String appPath = ApplicationContextFactory.getInstance().getApplicationContext().getAppPath();
		String htmlDir = CommonServiceProperties.getString(MAIL_TEMLATE_DIR);
		String text = parseFreemarkerTemplate(StringUtils.cleanPath(appPath + htmlDir), templateFileName, map);
		this.sendMail(to, cc, from, subject, text, isHTML, attachments, priority, saveStore);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.MailService#send(java.lang .String[], java.lang.String[], java.lang.String, java.lang.String,
	 * java.lang.String, boolean, java.util.List, int, boolean)
	 */
	public void sendMail(String[] to, String[] cc, String from, String subject, String text, boolean isHTML, List attachments, int priority, boolean saveStore) throws CommonServiceException {

		try {

			//
			log.debug("Mail send prepared.");

			//
			ContextFactory contextFactory = ContextFactory.getInstance();
			javax.naming.Context context = contextFactory.getContext();
			javax.mail.Session mail_session = (javax.mail.Session) context.lookup(CommonServiceProperties.getString(SEND_SESSION_JNDI));

			//
			MimeMessage msg = new MimeMessage(mail_session);

			// 메일 발송일
			msg.setSentDate(new java.util.Date());

			// 메일 수신자
			for (int i = 0; to != null && i < to.length; i++) {
				try {
					msg.addRecipient(Message.RecipientType.TO, new InternetAddress(to[i]));
				} catch (Exception ex) {
					log.error("Mail add recipient[to] error: address=[" + to[i] + "]", ex);
				}
			}

			// 메일 수신 참조자
			for (int i = 0; cc != null && i < cc.length; i++) {
				try {
					msg.addRecipient(Message.RecipientType.CC, new InternetAddress(cc[i]));
				} catch (Exception ex) {
					log.error("Mail add recipient[cc] error: address=[" + cc[i] + "]", ex);
				}
			}

			// 메일 발송자
			msg.setFrom(new InternetAddress(from));

			// 메일 제목
			String encode_subject = MimeUtility.encodeText(subject, MAIL_ENCODING, "B");
			msg.setSubject(encode_subject);

			// 
			Multipart multipart = new MimeMultipart();

			// 메세지
			BodyPart messageBodyPart = new MimeBodyPart();
			if (isHTML) {
				messageBodyPart.setContent(text, "text/html; charset=" + MAIL_ENCODING);
			} else {
				messageBodyPart.setText(text);
			}
			multipart.addBodyPart(messageBodyPart);

			// 첨부파일
			if (attachments != null && attachments.size() > 0) {
				for (int i = 0; i < attachments.size(); i++) {
					String filePath = (String) attachments.get(i);
					log.debug("Attatch file : path=[" + filePath + "]");

					File attatchFile = new java.io.File(filePath);
					String attatchName = attatchFile.getName();
					messageBodyPart = new MimeBodyPart();
					DataSource source = new FileDataSource(attatchFile);
					messageBodyPart.setDataHandler(new DataHandler(source));
					messageBodyPart.setFileName(attatchName);
					multipart.addBodyPart(messageBodyPart);
				}
			}

			// 메일 내용
			msg.setContent(multipart);

			// 전송
			Transport.send(msg);

			log.info("Mail sended successfully : to=" + Arrays.toString(to) + ", cc=" + Arrays.toString(cc) + ", subject=[" + subject + "]");

			// 발송 메일 저장 여부
			if (saveStore) {
				Store store = mail_session.getStore();
				store.connect();
				Folder f = store.getFolder(CommonServiceProperties.getString(MAIL_STORE_NAME));
				if (!f.exists()) {
					f.create(Folder.HOLDS_MESSAGES);
				}
				f.appendMessages(new Message[] { msg });
			}

		} catch (NamingException ex) {
			String msg = "Mail service send naming error : " + ex.getMessage();
			log.error(msg, ex);
			throw new CommonServiceException(msg, ex);
		} catch (AddressException ex) {
			String msg = "Mail service send address error : " + ex.getMessage();
			log.error(msg, ex);
			throw new CommonServiceException(msg, ex);
		} catch (MessagingException ex) {
			String msg = "Mail service send messaging error : " + ex.getMessage();
			log.error(msg, ex);
			throw new CommonServiceException(msg, ex);
		} catch (Exception ex) {
			String msg = "Mail service send error : " + ex.getMessage();
			log.error(msg, ex);
			throw new CommonServiceException(msg, ex);
		}

	}

	/**
	 * 프리메이커 HTML 메일 템플릿을 파싱한다.
	 * 
	 * @param dir
	 * @param fileName
	 * @param dataModel
	 * @return
	 * @throws Exception
	 */
	private String parseFreemarkerTemplate(String dir, String fileName, Map dataModel) throws CommonServiceException {
		try {
			//
			String domain = CommonServiceProperties.getString(MAIL_DOMAIN);
			dataModel.put("domain", domain);
			//
			Configuration configuration = new Configuration();
			configuration.setDirectoryForTemplateLoading(new java.io.File(dir));
			configuration.setTemplateUpdateDelay(0);
			configuration.setTemplateExceptionHandler(TemplateExceptionHandler.DEBUG_HANDLER);
			configuration.setObjectWrapper(ObjectWrapper.BEANS_WRAPPER);
			configuration.setDefaultEncoding("UTF-8");

			Template template = configuration.getTemplate(fileName);
			StringWriter output = new StringWriter();
			template.process(dataModel, output);

			return output.toString();

		} catch (Exception ex) {
			throw new CommonServiceException("Freemarker html mail template parsing error : " + ex.getMessage(), ex);
		}
	}

}
