/*
 * JDesigner (R) Web Application Development Platform 
 * Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 * http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jdesigner.platform.data;

import java.math.BigDecimal;
import java.sql.Clob;
import java.sql.Date;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

import org.jdesigner.platform.util.Assert;
import org.jdesigner.platform.util.DateUtils;
import org.jdesigner.platform.util.StringUtils;

/**
 * 
 * <B>Data Map</B>
 * 
 * @author leesangboo
 * 
 */
public class DataMap extends HashMap implements Map {

	private static final long serialVersionUID = 2505696114750248784L;

	/**
	 * 데이터 맵 생성자
	 */
	public DataMap() {
		super();
	}

	/**
	 * 데이터 맵 생성자
	 * 
	 * @param map
	 */
	public DataMap(Map map) {
		super(map);
	}

	/**
	 * key에 해당하는 String 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public String getString(String key) {
		return getString(key, Constants.IS_NULL_REPLACE_STRING);
	}

	/**
	 * key에 해당하는 String 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 *            - null일경우 덮어쓸 값
	 * @return
	 */
	public String getString(String key, String rpl) {
		if (this.containsKey(key) && this.get(key) != null) {
			return this.get(key).toString();
		} else {
			return rpl;
		}
	}

	/**
	 * key에 해당하는 int 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public int getInt(String key) {
		return org.apache.commons.lang.math.NumberUtils.toInt(this.get(key).toString());
	}

	/**
	 * key에 해당하는 long 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public long getLong(String key) {
		return org.apache.commons.lang.math.NumberUtils.toLong(this.get(key).toString());
	}

	/**
	 * key에 해당하는 float 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public float getFloat(String key) {
		return org.apache.commons.lang.math.NumberUtils.toFloat(this.get(key).toString());
	}

	/**
	 * key에 해당하는 BigDecimal 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public BigDecimal getBigDecimal(String key) {
		try {
			Assert.isInstanceOf(BigDecimal.class, this.get(key), "DataMap getting BigDecimal data error.");
			return (BigDecimal) this.get(key);
		} catch (Exception ex) {
			throw new DataException(ex.getMessage());
		}
	}

	/**
	 * key에 해당하는 Date 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Date getDate(String key) {
		try {
			Assert.isInstanceOf(Timestamp.class, this.get(key), "DataMap getting Date data error.");
			return new Date(((Timestamp) this.get(key)).getTime());
		} catch (Exception ex) {
			throw new DataException(ex.getMessage());
		}
	}

	/**
	 * key에 해당하는 Date 데이터 타입의 값을 ISO 형태의 날짜 String 값으로 변환하여 반환한다.
	 * 
	 * @param key
	 * @return
	 * @throws DataException
	 */
	public String getDateToString(String key) {
		return DateUtils.dateToISO(getDate(key), true);
	}

	/**
	 * key에 해당하는 Timestamp 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Timestamp getTimestamp(String key) {
		try {
			Assert.isInstanceOf(Timestamp.class, this.get(key), "DataMap getting Timestamp data error.");
			return (Timestamp) this.get(key);
		} catch (Exception ex) {
			throw new DataException(ex.getMessage());
		}
	}

	/**
	 * key에 해당하는 Date 데이터 타입의 값을 ISO 형태의 날짜 String 값으로 변환하여 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public String getTimestampToString(String key) {
		return DateUtils.timestampToISO(getTimestamp(key), true);
	}

	/**
	 * Timestamp 값을 Date String으로 변환하여 다시 저장한다.
	 * 
	 * @param key
	 */
	public void toTimestampToDateString(String key) {
		super.put(key, DateUtils.timestampToString(DataUtils.getDatePattern(), getTimestamp(key)));
	}

	/**
	 * Timestamp 값을 Date Time String으로 변환하여 다시 저장한다.
	 * 
	 * @param key
	 */
	public void toTimestampToDateTimeString(String key) {
		super.put(key, DateUtils.timestampToString(DataUtils.getDateTimePattern(), getTimestamp(key)));
	}

	/**
	 * key에 해당하는 Clob 데이터 타입의 값을 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Clob getClob(String key) {
		try {
			Assert.isInstanceOf(Clob.class, this.get(key), "DataMap getting CLOB data error.");
			return (Clob) this.get(key);
		} catch (Exception ex) {
			throw new DataException(ex.getMessage());
		}
	}

	/**
	 * String 형의 값을 Number 형으로 변환한다.
	 * 
	 * @param key
	 *            - Key
	 * @param clazz
	 *            - 변경할 데이터 타입 클래스
	 */
	public void convertNumberType(String key, Class clazz) {
		try {
			if (StringUtils.isNotEmpty(getString(key))) {
				String value = getString(key);
				if (clazz == Float.class) {
					this.put(key, new Float(Float.parseFloat(value)));
				} else if (clazz == Long.class) {
					this.put(key, new Long(Long.parseLong(value)));
				} else if (clazz == Double.class) {
					this.put(key, new Double(Double.parseDouble(value)));
				} else if (clazz == Integer.class) {
					this.put(key, new Integer(Integer.parseInt(value)));
				} else if (clazz == BigDecimal.class) {
					this.put(key, new BigDecimal(value));
				} else {
					throw new UnsupportedOperationException("Unsupport data type : class=[" + clazz + "]");
				}
			}
		} catch (Exception ex) {
			throw new DataException("Change number data type error key [" + key + "] : " + ex.getMessage(), ex);
		}
	}

}
