/*
 * JDesigner (R) Web Application Development Platform 
 * Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 * http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jdesigner.platform.data;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.jdesigner.platform.util.DateUtils;
import org.jdesigner.platform.util.FileUtils;

/**
 * <B>InputMap</B>
 * 
 * <pre> Map 의 value 유형이 String, String[]만 허용되는 Parameter Map Wrapping Class입니다. </pre>
 * 
 * @author leesangboo
 * 
 */
public class ParameterMap extends HashMap implements Map {

	private static final long serialVersionUID = 2505696114750248784L;

	/**
	 * Input 생성자
	 * 
	 * @param map
	 */
	public ParameterMap(Map map) {
		super(map);
	}

	/**
	 * Key에 해당하는 String 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public String getString(String key) {
		return getString(key, Constants.IS_NULL_REPLACE_STRING);
	}

	/**
	 * Key에 해당하는 String 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 *            - null일경우 덮어쓰기할 값
	 * @return
	 */
	public String getString(String key, String rpl) {
		if (this.containsKey(key) && this.get(key).getClass().isArray()) { // 
			String[] array = (String[]) this.get(key);
			if (array != null && array.length > 0) {
				return array[0];
			} else {
				return rpl;
			}
		} else { // not Array (Defalut)
			if (this.get(key) != null) {
				return this.get(key).toString();
			} else {
				return rpl;
			}
		}
	}

	/**
	 * Key에 해당하는 String[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public String[] getStringArray(String key) {
		if (this.containsKey(key)) {
			String[] array = (String[]) this.get(key);
			return array;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 int 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public int getInt(String key) {
		return org.apache.commons.lang.math.NumberUtils.toInt(getString(key));
	}

	/**
	 * Key에 해당하는 int 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public int getInt(String key, int rpl) {
		return org.apache.commons.lang.math.NumberUtils.toInt(getString(key), rpl);
	}

	/**
	 * Key에 해당하는 int[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public int[] getIntArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			int[] returnArray = new int[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = org.apache.commons.lang.math.NumberUtils.toInt(strArray[i]);
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 long 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public long getLong(String key) {
		return org.apache.commons.lang.math.NumberUtils.toLong(getString(key));
	}

	/**
	 * Key에 해당하는 long 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public long getLong(String key, long rpl) {
		return org.apache.commons.lang.math.NumberUtils.toLong(getString(key), rpl);
	}

	/**
	 * Key에 해당하는 long[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public long[] getLongArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			long[] returnArray = new long[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = org.apache.commons.lang.math.NumberUtils.toLong(strArray[i]);
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 float 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public float getFloat(String key) {
		return org.apache.commons.lang.math.NumberUtils.toFloat(getString(key));
	}

	/**
	 * Key에 해당하는 float[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public float[] getFloatArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			float[] returnArray = new float[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = org.apache.commons.lang.math.NumberUtils.toFloat(strArray[i]);
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 double 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public double getDouble(String key, double rpl) {
		return org.apache.commons.lang.math.NumberUtils.toDouble(getString(key), rpl);
	}

	/**
	 * Key에 해당하는 double 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public double getDouble(String key) {
		return org.apache.commons.lang.math.NumberUtils.toDouble(getString(key));
	}

	/**
	 * Key에 해당하는 double[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public double[] getDoubleArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			double[] returnArray = new double[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = org.apache.commons.lang.math.NumberUtils.toDouble(strArray[i]);
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 BigDecimal 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public BigDecimal getBigDecimal(String key) {
		return new BigDecimal((getString(key)));
	}

	/**
	 * Key에 해당하는 BigDecimal 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public BigDecimal getBigDecimal(String key, String rpl) {
		return new BigDecimal((getString(key, (rpl + ""))));
	}

	/**
	 * Key에 해당하는 BigDecimal[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public BigDecimal[] getBigDecimalArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			BigDecimal[] returnArray = new BigDecimal[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = new BigDecimal((strArray[i]));
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 Date 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Date getDate(String key) throws DataException {
		try {
			return DateUtils.stringToDate(DataUtils.getDatePattern(), getString(key));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Date 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public Date getDate(String key, String rpl) throws DataException {
		try {
			return DateUtils.stringToDate(DataUtils.getDatePattern(), getString(key, rpl));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Date[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Date[] getDateArray(String key) throws DataException {
		try {
			String[] strArray = getStringArray(key);
			if (strArray != null && strArray.length > 0) {
				Date[] returnArray = new Date[strArray.length];
				for (int i = 0; i < strArray.length; i++) {
					returnArray[i] = DateUtils.stringToDate(DataUtils.getDatePattern(), strArray[i]);
				}
				return returnArray;
			} else {
				return null;
			}
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Time 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Time getTime(String key) throws DataException {
		try {
			return DateUtils.stringToTime(DataUtils.getTimePattern(), getString(key));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Time 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public Time getTime(String key, String rpl) throws DataException {
		try {
			return DateUtils.stringToTime(DataUtils.getTimePattern(), getString(key, rpl));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Time[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Time[] getTimeArray(String key) throws DataException {
		try {
			String[] strArray = getStringArray(key);
			if (strArray != null && strArray.length > 0) {
				Time[] returnArray = new Time[strArray.length];
				for (int i = 0; i < strArray.length; i++) {
					returnArray[i] = DateUtils.stringToTime(DataUtils.getTimePattern(), strArray[i]);
				}
				return returnArray;
			} else {
				return null;
			}
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Timestamp 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Timestamp getTimestamp(String key) throws DataException {
		try {
			return DateUtils.stringToTimestamp(DataUtils.getDateTimePattern(), getString(key));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Timestamp 파라메터를 반환한다.
	 * 
	 * @param key
	 * @param rpl
	 * @return
	 */
	public Timestamp getTimestamp(String key, String rpl) throws DataException {
		try {
			return DateUtils.stringToTimestamp(DataUtils.getDateTimePattern(), getString(key, rpl));
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 Timestamp[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public Timestamp[] getTimestampArray(String key) throws DataException {
		try {
			String[] strArray = getStringArray(key);
			if (strArray != null && strArray.length > 0) {
				Timestamp[] returnArray = new Timestamp[strArray.length];
				for (int i = 0; i < strArray.length; i++) {
					returnArray[i] = DateUtils.stringToTimestamp(DataUtils.getDateTimePattern(), strArray[i]);
				}
				return returnArray;
			} else {
				return null;
			}
		} catch (Exception ex) {
			throw new DataException(ex.getMessage(), ex);
		}
	}

	/**
	 * Key에 해당하는 File 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public File getFile(String key) {
		return new File((getString(key)));
	}

	/**
	 * Key에 해당하는 File[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 */
	public File[] getFileArray(String key) {
		String[] strArray = getStringArray(key);
		if (strArray != null && strArray.length > 0) {
			File[] returnArray = new File[strArray.length];
			for (int i = 0; i < strArray.length; i++) {
				returnArray[i] = new File(strArray[i]);
			}
			return returnArray;
		} else {
			return null;
		}
	}

	/**
	 * Key에 해당하는 byte[] 파라메터를 반환한다.
	 * 
	 * @param key
	 * @return
	 * @throws IOException
	 */
	public byte[] getBytes(String key) throws IOException {
		return FileUtils.getBytesFromFile(new File((getString(key))));
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void put(String key, int value) {
		this.put(key, new Integer(value));
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void put(String key, long value) {
		this.put(key, new Long(value));
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void put(String key, float value) {
		this.put(key, new Float(value));
	}

}
