/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.parser;

import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.cache.Cache;
import org.jdesigner.platform.cache.CacheFactory;
import org.jdesigner.platform.cache.impl.LRUCache;
import org.jdesigner.platform.context.ApplicationConfigUtils;
import org.jdesigner.platform.db.query.NamedQuery;
import org.jdesigner.platform.db.query.Query;

/**
 * <B>SQL Factory</B>
 * 
 * @author leesangboo
 * 
 */
public class SQLFactory {

	private static final Log log = LogFactory.getLog(SQLFactory.class);

	private static final String SQL_CACHE_KEY = "platform.db.sql.cache";
	private static final String CACHE_ID = "platform.db.sql.cache.sql.xml";
	private static final int CACHE_SIZE = 1000;
	private static final String QUERY_XML_BASE_PATH_KEY = "platform.db.query.xml.basepath";

	private Cache cache;

	private SQLStringExtractor extractor;

	private static class SQLFactoryHolder {
		static SQLFactory instance = new SQLFactory();
	}

	public static SQLFactory getInstance() {
		return SQLFactoryHolder.instance;
	}

	public SQLFactory() {
		//
		Properties config = new Properties();
		config.put(LRUCache.CONFIG_MAX_ENTRIES_KEY, String.valueOf(CACHE_SIZE));
		cache = CacheFactory.createCache(CACHE_ID, Cache.LRU, config);
		//
		extractor = new SQLStringExtractor();
	}

	/**
	 * SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 */
	public String getSQL(Query query) throws QueryParsingException {

		boolean useCache = ApplicationConfigUtils.getBoolean(SQL_CACHE_KEY);
		String xmlPath = ApplicationConfigUtils.getProperty(QUERY_XML_BASE_PATH_KEY) + query.getPath();

		if (useCache) { // 캐슁 사용 여부
			if (!cache.hasValue(xmlPath)) {
				String sql = extractor.extraction(xmlPath);
				cache.put(xmlPath, sql);
				log.info("XML SQL cached : key=[" + xmlPath + "], cachedSize=[" + getSQLMap().size() + "]");
				return sql;
			} else {
				log.debug("XML SQL cached return : key=[" + xmlPath + "]");
				return (String) cache.get(xmlPath);
			}
		} else {
			return extractor.extraction(xmlPath);
		}
	}

	/**
	 * SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 */
	public String getSQL(NamedQuery query) throws QueryParsingException {

		boolean useCache = ApplicationConfigUtils.getBoolean(SQL_CACHE_KEY);
		String xmlPath = ApplicationConfigUtils.getProperty(QUERY_XML_BASE_PATH_KEY) + query.getPath();

		if (useCache) { // 캐슁 사용 여부
			if (!cache.hasValue(xmlPath)) {
				String sql = extractor.extraction(xmlPath);
				cache.put(xmlPath, sql);
				log.info("XML Named SQL cached : key=[" + xmlPath + "], cachedSize=[" + getSQLMap().size() + "]");
				return sql;
			} else {
				log.debug("XML Named SQL cached return : key=[" + xmlPath + "]");
				return (String) cache.get(xmlPath);
			}
		} else {
			return extractor.extraction(xmlPath);
		}
	}

	/**
	 * 캐쉬된 SQL 맵을 반환한다.
	 * 
	 * @return
	 */
	public Map getSQLMap() throws QueryParsingException {
		return cache.toMap();
	}

}
