/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.processor.impl;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.data.DataMap;
import org.jdesigner.platform.data.Page;
import org.jdesigner.platform.data.Tree;
import org.jdesigner.platform.data.TreeNode;
import org.jdesigner.platform.db.domain.UpdateResult;
import org.jdesigner.platform.db.processor.NamedQueryProcessor;
import org.jdesigner.platform.db.processor.QueryProcessingException;
import org.jdesigner.platform.db.processor.QueryValidateException;
import org.jdesigner.platform.db.query.NamedQuery;

/**
 * NamedQueryProcessorImpl
 * 
 * @author leesangboo
 * 
 */
public class NamedQueryProcessorImpl extends SQLProcessor implements NamedQueryProcessor {

	private static final Log log = LogFactory.getLog(NamedQueryProcessorImpl.class);

	private NamedQuery query;

	public NamedQueryProcessorImpl(NamedQuery query) {
		this.query = query;
		this.validate();
		log.info("Query : " + query.toString());
	}

	/**
	 * Query 검증
	 * 
	 */
	private void validate() {
		if (query.getPath() == null && query.getPath().equals("")) {
			throw new QueryValidateException("NamedQuery [path] value is null, this is required value.");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.NamedQueryProcessor#selectPage(int, int, java.lang.String, boolean)
	 */
	public Page selectPage(int start, int range, String column, boolean desc) throws QueryProcessingException {
		try {
			return selectPage(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query), start, range,
					column, desc);
		} catch (Exception ex) {
			String msg = "[selectPage] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.processor.NamedQueryProcessor#selectTree(java .lang.String, java.lang.String)
	 */
	public Tree selectTree(String codeColumnName, String parentCodeColumnName) throws QueryProcessingException {
		try {
			Tree tree = null;
			List result = selectList(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query));
			if (result != null) {
				tree = new Tree();
				for (int i = 0; result != null && i < result.size(); i++) {
					DataMap data = (DataMap) result.get(i);
					String code = data.getString(codeColumnName);
					String parentCode = data.getString(parentCodeColumnName);
					if (StringUtils.isEmpty(code)) {
						throw new QueryProcessingException("Not found tree [code] value in row data.");
					}
					if (StringUtils.isEmpty(parentCode)) {
						throw new QueryProcessingException("Not found tree [parentCode] value in row data.");
					}
					tree.add(new TreeNode(code, parentCode, data));
				}
			}
			return tree;
		} catch (Exception ex) {
			String msg = "[selectList] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.NamedQueryProcessor#selectList(org.architecture .common.db.NamedQuery, java.lang.Map)
	 */
	public List selectList() throws QueryProcessingException {
		try {
			return selectList(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query));
		} catch (Exception ex) {
			String msg = "[selectList] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.NamedQueryProcessor#selectMap(org.architecture .common.db.NamedQuery, java.lang.Map)
	 */
	public Map selectMap() throws QueryProcessingException {
		try {
			return selectMap(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query));
		} catch (Exception ex) {
			String msg = "[selectMap] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.NamedQueryProcessor#selectObject()
	 */
	public Object selectObject() throws QueryProcessingException {
		try {
			return selectObject(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query));
		} catch (Exception ex) {
			String msg = "[selectObject] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.NamedQueryProcessor#update()
	 */
	public UpdateResult update() throws QueryProcessingException {
		try {
			return update(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), super.getParsedDynamicSQL(query), super.createMapParameters(query));
		} catch (Exception ex) {
			String msg = "[update] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

}
