/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.table;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.jdesigner.platform.context.ApplicationConfigUtils;
import org.jdesigner.platform.db.connection.ConnectionUtils;
import org.jdesigner.platform.db.provider.RDBMSProvider;
import org.jdesigner.platform.db.provider.RDBMSProviderFactory;
import org.jdesigner.platform.db.table.model.Column;
import org.jdesigner.platform.db.table.model.TableLayout;
import org.jdesigner.platform.db.table.rdbms.TableTypeMaker;
import org.jdesigner.platform.util.StringUtils;

/**
 * <B>TableLayoutLoader</B>
 * 
 * @author leesangboo
 * 
 */
public class TableLayoutLoader {

	private static final String TABLE_SCHEMA_PATTERN_KEY = "platform.db.table.schema";

	/**
	 * 테이블 레이아웃을 반환한다.
	 * 
	 * @param dm
	 *            - 데이터베이스 메터 데이터
	 * @param tableName
	 *            - 테이블 명
	 * @param pks
	 *            - 사용자 지정 프라이머리 키 배열
	 * @return
	 * @throws SQLException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public TableLayout getTableLayout(Connection connection, DatabaseMetaData dm, String tableName, String[] pks) throws TableSQLException {
		TableLayout tableLayout = new TableLayout();
		ResultSet resultSet = null;
		try {

			//
			String schemaPattern = null;
			if (tableName.indexOf(".") > -1) {
				int dotIndex = tableName.indexOf(".");
				schemaPattern = tableName.substring(0, dotIndex);
				tableName = tableName.substring(dotIndex + 1);
			}
			;

			//
			if (StringUtils.isEmpty(schemaPattern)) {
				String defaultSchemaName = ApplicationConfigUtils.getProperty(TABLE_SCHEMA_PATTERN_KEY);
				if (StringUtils.isNotEmpty(defaultSchemaName)) {
					schemaPattern = defaultSchemaName;
				}
			}

			//
			resultSet = dm.getTables(null, schemaPattern, tableName, new String[] { "TABLE" });

			//
			if (resultSet.next()) {
				String categoryName = resultSet.getString("TABLE_CAT");
				String schemaName = resultSet.getString("TABLE_SCHEM");

				ArrayList primaryKeys = (pks == null || pks.length < 1) ? getPrimaryKeys(dm, categoryName, schemaName, tableName) : // 사용자가
						// 정의한
						// 프라이머리
						// 키
						// 사용
						StringUtils.array2ArrayList(pks);// 테이블의 기본 프라이머리 키
				// 사용
				ArrayList columns = getColumns(connection, dm, categoryName, schemaName, tableName);

				tableLayout.setCategory(categoryName);
				tableLayout.setSchema(schemaName);
				tableLayout.setTableName(tableName);
				tableLayout.setPrimaryKeys(primaryKeys);
				tableLayout.setColumns(columns);
			} else {
				throw new TableSQLException("Not found table from database metadata : schemaPattern=[" + schemaPattern + "], tableName=[" + tableName + "]");
			}

			return tableLayout;
		} catch (Exception ex) {
			throw new TableSQLException("TableLayoutLoader getting table layout exception : " + ex.getMessage(), ex);
		} finally {
			ConnectionUtils.closeResultSet(resultSet);
		}
	}

	/**
	 * 컬럼 목록을 반환한다.
	 * 
	 * @param dm
	 * @param categoryName
	 * @param schemaName
	 * @param tableName
	 * @return
	 * @throws SQLException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	private ArrayList getColumns(Connection connection, DatabaseMetaData dm, String categoryName, String schemaName, String tableName) throws SQLException, InstantiationException,
			IllegalAccessException {

		ArrayList columns = new ArrayList();
		ResultSet resultSet = null;

		try {

			//
			RDBMSProvider provider = RDBMSProviderFactory.getInstance().getRDBMSProvider(connection);
			TableTypeMaker typeMaker = provider.getTableTypeMaker();

			//
			resultSet = dm.getColumns(categoryName, schemaName, tableName, "%");

			//
			int columnCount = 0;
			while (resultSet.next()) {
				//
				Column column = new Column();
				column.setColumnName(resultSet.getString("COLUMN_NAME"));
				column.setColumnSize(resultSet.getInt("COLUMN_SIZE"));
				column.setColumnDef(resultSet.getString("COLUMN_DEF"));
				column.setTypeName(resultSet.getString("TYPE_NAME"));
				column.setNullable(resultSet.getBoolean("NULLABLE"));
				column.setDecimalDigits(resultSet.getInt("DECIMAL_DIGITS"));
				column.setJavaType(typeMaker.getType(column));
				//
				columns.add(column);
				//
				columnCount++;
			}
			if (columnCount == 0) {
				throw new TableSQLException("Not found database table columns from metadata : " + tableName);
			}

			return columns;

		} catch (Exception ex) {
			throw new TableSQLException("TableLayoutLoader getting columns exception : " + ex.getMessage(), ex);
		} finally {
			ConnectionUtils.closeResultSet(resultSet);
		}

	}

	/**
	 * 프라이머리 키 목록을 반환한다.
	 * 
	 * @param dm
	 * @param categoryName
	 * @param schemaName
	 * @param tableName
	 * @return
	 * @throws SQLException
	 */
	private ArrayList getPrimaryKeys(DatabaseMetaData dm, String categoryName, String schemaName, String tableName) throws SQLException {
		ArrayList primarKeys = new ArrayList();
		ResultSet resultSet = null;

		try {
			resultSet = dm.getPrimaryKeys(categoryName, schemaName, tableName);
			while (resultSet.next()) {
				primarKeys.add(resultSet.getString("COLUMN_NAME"));
				// (resultSet.getString("PK_NAME"));
				// (resultSet.getInt("KEY_SEQ"));
			}
			return primarKeys;
		} catch (Exception ex) {
			throw new TableSQLException("TableLayoutLoader getting primary keys exception : " + ex.getMessage(), ex);
		} finally {
			ConnectionUtils.closeResultSet(resultSet);
		}
	}

}
