/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.transaction;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.db.connection.ConnectionException;
import org.jdesigner.platform.db.connection.ConnectionManager;
import org.jdesigner.platform.db.connection.ConnectionUtils;

/**
 * BaseTransaction
 * 
 * @author leesangboo
 * 
 */
abstract class BaseTransaction {

	protected static Log log = LogFactory.getLog(BaseTransaction.class);

	protected Map changedConnectionMap;

	protected Connection connection;

	protected String previousDataSourceJNDIName;

	protected String dataSourceJNDIName;

	protected boolean active;

	protected boolean commited;

	public Connection getConnection() throws TransactionException {
		try {
			if (connection == null || connection.isClosed()) {
				try {
					//
					connection = new ConnectionManager().getConnection(dataSourceJNDIName);
					//
					if (connection.getAutoCommit()) {
						connection.setAutoCommit(false);
					}
					//connection.setTransactionIsolation(Connection.TRANSACTION_SERIALIZABLE);
					log.info("Create database connection : dataSourceJNDIName=[" + dataSourceJNDIName + "]");
					//
				} catch (Exception e) {
					throw new ConnectionException("Failed open connection : " + e.getMessage(), e);
				}
			}
			return connection;
		} catch (SQLException ex) {
			String msg = "Cannot getting connection : " + ex.getMessage();
			log.error(msg, ex);
			throw new TransactionException(msg, ex);
		}
	}

	public void changeConnection(String dataSourceJNDIName) throws TransactionException {
		try {
			//이전 데이터소스와 같은지 비교
			if (!this.dataSourceJNDIName.equals(dataSourceJNDIName)) {

				//
				this.previousDataSourceJNDIName = dataSourceJNDIName;
				this.dataSourceJNDIName = dataSourceJNDIName;

				//이미 변경된 커넥션 맵에 존재하는지 확인
				if (changedConnectionMap != null && changedConnectionMap.containsKey(dataSourceJNDIName)) {
					//
					if (connection != null || connection.isClosed()) {
						//커넥션 변경시, 기존 커넥션을  변경 맵에 저장한다.
						if (changedConnectionMap == null) {
							changedConnectionMap = Collections.synchronizedMap(new HashMap());
						}
						changedConnectionMap.put(dataSourceJNDIName, connection);
					}
					getConnection();
					log.info("Connection have been changed : dataSourceJNDIName=[" + dataSourceJNDIName + "], previous=[" + previousDataSourceJNDIName + "]");
				} else {
					connection = (Connection) changedConnectionMap.get(dataSourceJNDIName);
					log.info("Return to map changes in the stored connection : dataSourceJNDIName=[" + dataSourceJNDIName + "], previous=[" + previousDataSourceJNDIName + "]");
				}
			} else {
				log.warn("Change the datasource is the same as the previous datasource.");
			}
		} catch (SQLException ex) {
			String msg = "Cannot getting connection : " + ex.getMessage();
			log.error(msg, ex);
			throw new TransactionException(msg, ex);
		}
	}

	protected void closeChangedConnections() throws TransactionException {
		if (changedConnectionMap != null && !changedConnectionMap.isEmpty()) {
			Object[] keys = changedConnectionMap.keySet().toArray();
			for (int i = 0; i < keys.length; i++) {
				log.info("Close changed connection : dataSourceJNDIName=[" + dataSourceJNDIName + "]");
				String key = (String) keys[i];
				ConnectionUtils.closeConnection((Connection) changedConnectionMap.get(key));
			}
		}
	}

	public Map getChangedConnectionMap() throws TransactionException {
		return changedConnectionMap;
	}

	public boolean isActive() {
		return active;
	}

	public boolean isCommited() {
		return commited;
	}

}
