package org.jdesigner.platform.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 파일 유틸
 * 
 * @author leesangboo
 */
public class FileUtils extends org.apache.commons.io.FileUtils {

	private static final Log log = LogFactory.getLog(FileUtils.class);

	public FileUtils() {
		//
	}

	/**
	 * Get an InputStream from specified resource uri. URI can be absolute or relative filename, or complete URL.
	 */
	public final static InputStream getResource(String uri) throws IOException {
		try {
			return new URL(uri).openStream();
		} catch (MalformedURLException mue) {
			return new FileInputStream(uri);
		}
	}

	/**
	 * 
	 * @param file
	 * @return
	 */
	public final static File checkBackupDirectory(File file) {
		File backupDirectory = new File(file.getParent() + System.getProperty("file.separator") + "_backup");

		if (!backupDirectory.exists()) {
			backupDirectory.mkdirs();
		}

		return backupDirectory;
	}

	/**
	 * 
	 * @param path
	 * @return
	 */
	public final static File createFile(String path) {
		File file = new File(path);

		try {
			file.createNewFile();
		} catch (IOException e) {
			log.error(e);
			e.printStackTrace();
		}

		return file;
	}

	/**
	 * 
	 * @param path
	 * @return
	 */
	public final static String[] dirList(String path) {
		return dirList(new File(path));
	}

	/**
	 * 
	 * @param path
	 * @return
	 */
	public final static String[] dirList(File path) {
		String[] list;

		list = path.list();

		return list;
	}

	/**
	 * 
	 * @param file
	 * @return
	 */
	public final static String readFile(File file) {
		try {
			BufferedReader in = new BufferedReader(new FileReader(file));

			String s = new String();
			StringBuffer buffer = new StringBuffer();

			while ((s = in.readLine()) != null) {
				buffer.append(s + "\n");
			}

			in.close();

			return buffer.toString();
		} catch (FileNotFoundException e) {
			log.warn("File not found");
			e.printStackTrace();
		} catch (IOException e) {
			log.error(e);
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * 
	 * @param file
	 * @param content
	 */
	public final static void write(File file, String content) {
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(file));
			out.write(content);
			out.close();
		} catch (FileNotFoundException e) {
			log.warn("File not found", e);
			e.printStackTrace();
		} catch (IOException e) {
			log.error(e);
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @param file
	 * @param content
	 */
	public final static void writeAndBackup(File file, String content) {
		try {
			DateFormat backupDF = new SimpleDateFormat("yyyyMMdd_hhmmss");

			File backupDirectory = checkBackupDirectory(file);

			File original = new File(file.getAbsolutePath());

			File backup = new File(backupDirectory, original.getName() + "." + backupDF.format(new Date()));

			if (log.isDebugEnabled()) {
				log.debug("Backup file is " + backup);
			}

			original.renameTo(backup);

			BufferedWriter out = new BufferedWriter(new FileWriter(file));

			out.write(content);
			out.close();
		} catch (FileNotFoundException e) {
			log.warn("File not found", e);
			e.printStackTrace();
		} catch (IOException e) {
			log.error(e);
			e.printStackTrace();
		}
	}

	/**
	 * 디스플레이를 위한 파일사이즈를 반환한다.
	 * 
	 * @param filesize
	 * @return
	 */
	public static String getDisplayFileSize(long filesize) {
		DecimalFormat df = new DecimalFormat(".##");
		String fSize = "";
		if ((filesize > 1024) && (filesize < 1024 * 1024)) {
			fSize = df.format((float) filesize / 1024).toString() + " KB";
		} else if (filesize >= 1024 * 1024) {
			fSize = df.format((float) filesize / (1024 * 1024)).toString() + " MB";
		} else {
			fSize = Long.toString(filesize) + " Bytes";
		}
		return fSize;
	}

	/**
	 * Returns the contents of the file in a byte array
	 * 
	 * @param file
	 *            File this method should read
	 * @return byte[] Returns a byte[] array of the contents of the file
	 */
	public static byte[] getBytesFromFile(File file) throws IOException {

		InputStream is = new FileInputStream(file);
		log.debug("FileInputStream is " + file);

		// Get the size of the file
		long length = file.length();
		log.debug("Length of " + file + " is " + length + "\n");

		/*
		 * You cannot create an array using a long type. It needs to be an int type. Before converting to an int type, check to ensure that file is
		 * not loarger than Integer.MAX_VALUE;
		 */
		if (length > Integer.MAX_VALUE) {
			log.debug("File is too large to process");
			return null;
		}

		// Create the byte array to hold the data
		byte[] bytes = new byte[(int) length];

		// Read in the bytes
		int offset = 0;
		int numRead = 0;
		while ((offset < bytes.length) && ((numRead = is.read(bytes, offset, bytes.length - offset)) >= 0)) {

			offset += numRead;

		}

		// Ensure all the bytes have been read in
		if (offset < bytes.length) {
			throw new IOException("Could not completely read file " + file.getName());
		}

		is.close();
		return bytes;

	}

}
